/*
 *  Copyright (c) 2014-present, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the BSD-style license found in the
 *  LICENSE file in the root directory of this source tree. An additional grant
 *  of patent rights can be found in the PATENTS file in the same directory.
 *
 */

/*
** This file is generated. Do not modify it manually!
*/

#include <osquery/events.h>
#include <osquery/logger.h>
#include <osquery/tables.h>


namespace osquery {


namespace tables {
osquery::QueryData genOpenSockets(QueryContext& context);
}

class processOpenSocketsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("pid", INTEGER_TYPE, ColumnOptions::INDEX),
      std::make_tuple("fd", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("socket", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("family", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("protocol", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("local_address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("remote_address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("local_port", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("remote_port", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genOpenSockets(context);

    return results;
  }

};


REGISTER(processOpenSocketsTablePlugin, "table", "process_open_sockets");


namespace tables {
osquery::QueryData genUserGroups(QueryContext& context);
}

class userGroupsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("gid", BIGINT_TYPE, ColumnOptions::INDEX),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genUserGroups(context);

    return results;
  }

};


REGISTER(userGroupsTablePlugin, "table", "user_groups");


namespace tables {
osquery::QueryData genDeviceFile(QueryContext& context);
}

class deviceFileTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("device", TEXT_TYPE, ColumnOptions::INDEX | ColumnOptions::REQUIRED),
      std::make_tuple("partition", TEXT_TYPE, ColumnOptions::INDEX | ColumnOptions::REQUIRED),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::ADDITIONAL),
      std::make_tuple("filename", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("inode", BIGINT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("gid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mode", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("block_size", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("atime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mtime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ctime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("hard_links", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genDeviceFile(context);

    return results;
  }

};


REGISTER(deviceFileTablePlugin, "table", "device_file");


namespace tables {
osquery::QueryData genEtcProtocols(QueryContext& context);
}

class etcProtocolsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("number", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("alias", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("comment", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genEtcProtocols(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(etcProtocolsTablePlugin, "table", "etc_protocols");


namespace tables {
osquery::QueryData genFirefoxAddons(QueryContext& context);
}

class firefoxAddonsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::ADDITIONAL),
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("identifier", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("creator", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("source_url", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("visible", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("active", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("disabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("autoupdate", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("native", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("location", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::USER_BASED |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genFirefoxAddons(context);

    return results;
  }

};


REGISTER(firefoxAddonsTablePlugin, "table", "firefox_addons");


namespace tables {
osquery::QueryData genDebPackages(QueryContext& context);
}

class debPackagesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("source", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("arch", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("revision", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genDebPackages(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(debPackagesTablePlugin, "table", "deb_packages");


namespace tables {
osquery::QueryData genLLDPNeighbors(QueryContext& context);
}

class lldpNeighborsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("interface", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("rid", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_id_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_sysname", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_sys_description", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_bridge_capability_available", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_bridge_capability_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_router_capability_available", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_router_capability_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_repeater_capability_available", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_repeater_capability_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_wlan_capability_available", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_wlan_capability_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_tel_capability_available", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_tel_capability_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_docsis_capability_available", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_docsis_capability_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_station_capability_available", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_station_capability_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_other_capability_available", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_other_capability_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chassis_mgmt_ips", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_id_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_description", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_ttl", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_mfs", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_aggregation_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_supported", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_mau_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_10baset_hd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_10baset_fd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_100basetx_hd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_100basetx_fd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_100baset2_hd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_100baset2_fd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_100baset4_hd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_100baset4_fd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_1000basex_hd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_1000basex_fd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_1000baset_hd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port_autoneg_1000baset_fd_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_device_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_mdi_supported", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_mdi_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_paircontrol_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_pairs", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_class", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_8023at_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_8023at_power_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_8023at_power_source", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_8023at_power_priority", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_8023at_power_allocated", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("power_8023at_power_requested", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("med_device_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("med_capability_capabilities", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("med_capability_policy", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("med_capability_location", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("med_capability_mdi_pse", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("med_capability_mdi_pd", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("med_capability_inventory", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("med_policies", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("vlans", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pvid", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ppvids_supported", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ppvids_enabled", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pids", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genLLDPNeighbors(context);

    return results;
  }

};


REGISTER(lldpNeighborsTablePlugin, "table", "lldp_neighbors");


namespace tables {
osquery::QueryData genSuidBin(QueryContext& context);
}

class suidBinTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("username", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("groupname", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("permissions", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genSuidBin(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(suidBinTablePlugin, "table", "suid_bin");


namespace tables {
osquery::QueryData genLoggedInUsers(QueryContext& context);
}

class loggedInUsersTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("user", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("tty", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("host", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("time", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pid", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genLoggedInUsers(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(loggedInUsersTablePlugin, "table", "logged_in_users");


namespace tables {
osquery::QueryData genVolumes(QueryContext& context);
}

class dockerVolumesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("driver", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mount_point", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genVolumes(context);

    return results;
  }

};


REGISTER(dockerVolumesTablePlugin, "table", "docker_volumes");


namespace tables {
osquery::QueryData genAptSrcs(QueryContext& context);
}

class aptSourcesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("base_uri", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("package_cache_file", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("release", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("component", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("maintainer", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("site", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genAptSrcs(context);

    return results;
  }

};


REGISTER(aptSourcesTablePlugin, "table", "apt_sources");


namespace tables {
osquery::QueryData genYara(QueryContext& context);
}

class yaraTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::INDEX | ColumnOptions::REQUIRED),
      std::make_tuple("matches", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("count", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sig_group", TEXT_TYPE, ColumnOptions::INDEX | ColumnOptions::ADDITIONAL),
      std::make_tuple("sigfile", TEXT_TYPE, ColumnOptions::INDEX | ColumnOptions::ADDITIONAL),
      std::make_tuple("strings", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("tags", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genYara(context);

    return results;
  }

};


REGISTER(yaraTablePlugin, "table", "yara");


namespace tables {
osquery::QueryData genIptables(QueryContext& context);
}

class iptablesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("filter_name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chain", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("policy", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("target", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("protocol", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("src_ip", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("src_mask", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("iniface", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("iniface_mask", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("dst_ip", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("dst_mask", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("outiface", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("outiface_mask", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("match", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("packets", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("bytes", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genIptables(context);

    return results;
  }

};


REGISTER(iptablesTablePlugin, "table", "iptables");


namespace tables {
osquery::QueryData genContainerProcesses(QueryContext& context);
}

class dockerContainerProcessesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::INDEX | ColumnOptions::REQUIRED),
      std::make_tuple("pid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cmdline", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("state", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("gid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("euid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("egid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("suid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sgid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("wired_size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("resident_size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("total_size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("start_time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("parent", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pgroup", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("threads", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("nice", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("user", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("time", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu", DOUBLE_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mem", DOUBLE_TYPE, ColumnOptions::DEFAULT),
    };
  }


  ColumnAliasSet columnAliases() const override {
    return {
      {"total_size", {"phys_footprint"}},
    };
  }

  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genContainerProcesses(context);

    return results;
  }

};


REGISTER(dockerContainerProcessesTablePlugin, "table", "docker_container_processes");


namespace tables {

class hardware_events {
 public:
  void genTable(RowYield& yield, QueryContext& context);
};
}

class hardwareEventsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("action", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("driver", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("vendor", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("vendor_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("model", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("model_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("serial", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("revision", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("eid", TEXT_TYPE, ColumnOptions::HIDDEN),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::EVENT_BASED |      TableAttributes::NONE;
  }

  bool usesGenerator() const override { return true; }

  void generator(RowYield& yield, QueryContext& context) override {
    if (EventFactory::exists(getName())) {
      auto subscriber = EventFactory::getEventSubscriber(getName());
      return subscriber->genTable(yield, context);
    } else {
      LOG(ERROR) << "Subscriber table missing: " << getName();
    }
  }

};


REGISTER(hardwareEventsTablePlugin, "table", "hardware_events");


namespace tables {
void genRpmPackageFiles(RowYield& yield, QueryContext& context);
}

class rpmPackageFilesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("package", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("username", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("groupname", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mode", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sha256", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  bool usesGenerator() const override { return true; }

  void generator(RowYield& yield, QueryContext& context) override {
    tables::genRpmPackageFiles(yield, context);
  }

};


REGISTER(rpmPackageFilesTablePlugin, "table", "rpm_package_files");


namespace tables {
osquery::QueryData genPortageKeywordSummary(QueryContext& context);
}

class portageKeywordsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("package", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("keyword", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mask", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("unmask", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genPortageKeywordSummary(context);

    return results;
  }

};


REGISTER(portageKeywordsTablePlugin, "table", "portage_keywords");


namespace tables {

class file_events {
 public:
  void genTable(RowYield& yield, QueryContext& context);
};
}

class fileEventsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("target_path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("category", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("action", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("transaction_id", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("inode", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("gid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mode", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("atime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mtime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ctime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("md5", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sha1", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sha256", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("hashed", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("eid", TEXT_TYPE, ColumnOptions::HIDDEN),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::EVENT_BASED |      TableAttributes::NONE;
  }

  bool usesGenerator() const override { return true; }

  void generator(RowYield& yield, QueryContext& context) override {
    if (EventFactory::exists(getName())) {
      auto subscriber = EventFactory::getEventSubscriber(getName());
      return subscriber->genTable(yield, context);
    } else {
      LOG(ERROR) << "Subscriber table missing: " << getName();
    }
  }

};


REGISTER(fileEventsTablePlugin, "table", "file_events");


namespace tables {
osquery::QueryData genCarves(QueryContext& context);
}

class carvesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sha256", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("status", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("carve_guid", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("carve", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::USER_BASED |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genCarves(context);

    return results;
  }

};


REGISTER(carvesTablePlugin, "table", "carves");


namespace tables {
osquery::QueryData genContainers(QueryContext& context);
}

class dockerContainersTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("image", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("image_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("command", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("created", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("state", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("status", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genContainers(context);

    return results;
  }

};


REGISTER(dockerContainersTablePlugin, "table", "docker_containers");


namespace tables {
osquery::QueryData genVolumeLabels(QueryContext& context);
}

class dockerVolumeLabelsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("key", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genVolumeLabels(context);

    return results;
  }

};


REGISTER(dockerVolumeLabelsTablePlugin, "table", "docker_volume_labels");


namespace tables {
osquery::QueryData genContainerNetworks(QueryContext& context);
}

class dockerContainerNetworksTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("network_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("endpoint_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("gateway", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ip_address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ip_prefix_len", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ipv6_gateway", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ipv6_address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ipv6_prefix_len", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mac_address", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genContainerNetworks(context);

    return results;
  }

};


REGISTER(dockerContainerNetworksTablePlugin, "table", "docker_container_networks");


namespace tables {
osquery::QueryData genAugeas(QueryContext& context);
}

class augeasTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("node", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("label", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::ADDITIONAL),
    };
  }

  std::vector<std::string> aliases() const override {
    return {
      "configurations",
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genAugeas(context);

    return results;
  }

};


REGISTER(augeasTablePlugin, "table", "augeas");


namespace tables {
osquery::QueryData genLastAccess(QueryContext& context);
}

class lastTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("username", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("tty", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pid", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("time", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("host", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genLastAccess(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(lastTablePlugin, "table", "last");


namespace tables {
osquery::QueryData genArpCache(QueryContext& context);
}

class arpCacheTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mac", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("interface", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("permanent", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genArpCache(context);

    return results;
  }

};


REGISTER(arpCacheTablePlugin, "table", "arp_cache");


namespace tables {
osquery::QueryData genCronTab(QueryContext& context);
}

class crontabTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("event", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("minute", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("hour", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("day_of_month", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("month", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("day_of_week", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("command", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genCronTab(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(crontabTablePlugin, "table", "crontab");


namespace tables {
osquery::QueryData genInfo(QueryContext& context);
}

class dockerInfoTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("containers", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("containers_running", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("containers_paused", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("containers_stopped", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("images", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("storage_driver", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("memory_limit", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("swap_limit", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("kernel_memory", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_cfs_period", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_cfs_quota", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_shares", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_set", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ipv4_forwarding", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("bridge_nf_iptables", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("bridge_nf_ip6tables", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("oom_kill_disable", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("logging_driver", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cgroup_driver", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("kernel_version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("os", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("os_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("architecture", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpus", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("memory", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("http_proxy", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("https_proxy", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("no_proxy", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("server_version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("root_dir", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genInfo(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(dockerInfoTablePlugin, "table", "docker_info");


namespace tables {
osquery::QueryData genContainerLabels(QueryContext& context);
}

class dockerContainerLabelsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("key", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genContainerLabels(context);

    return results;
  }

};


REGISTER(dockerContainerLabelsTablePlugin, "table", "docker_container_labels");


namespace tables {
osquery::QueryData genKernelModules(QueryContext& context);
}

class kernelModulesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("used_by", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("status", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("address", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genKernelModules(context);

    return results;
  }

};


REGISTER(kernelModulesTablePlugin, "table", "kernel_modules");


namespace tables {
osquery::QueryData genMDPersonalities(QueryContext& context);
}

class mdPersonalitiesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genMDPersonalities(context);

    return results;
  }

};


REGISTER(mdPersonalitiesTablePlugin, "table", "md_personalities");


namespace tables {
osquery::QueryData getUserSshKeys(QueryContext& context);
}

class userSshKeysTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::ADDITIONAL),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("encrypted", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::getUserSshKeys(context);

    return results;
  }

};


REGISTER(userSshKeysTablePlugin, "table", "user_ssh_keys");


namespace tables {
osquery::QueryData genUptime(QueryContext& context);
}

class uptimeTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("days", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("hours", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("minutes", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("seconds", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("total_seconds", BIGINT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genUptime(context);

    return results;
  }

};


REGISTER(uptimeTablePlugin, "table", "uptime");


namespace tables {
osquery::QueryData genCurl(QueryContext& context);
}

class curlTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("url", TEXT_TYPE, ColumnOptions::INDEX | ColumnOptions::REQUIRED),
      std::make_tuple("method", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("user_agent", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("response_code", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("round_trip_time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("bytes", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("result", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genCurl(context);

    return results;
  }

};


REGISTER(curlTablePlugin, "table", "curl");


namespace tables {
osquery::QueryData genUsers(QueryContext& context);
}

class usersTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("gid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uid_signed", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("gid_signed", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("username", TEXT_TYPE, ColumnOptions::ADDITIONAL),
      std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("directory", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("shell", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uuid", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genUsers(context);

    return results;
  }

};


REGISTER(usersTablePlugin, "table", "users");


namespace tables {
osquery::QueryData genProcesses(QueryContext& context);
}

class processesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("pid", BIGINT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cmdline", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("state", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cwd", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("root", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("gid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("euid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("egid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("suid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sgid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("on_disk", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("wired_size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("resident_size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("total_size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("user_time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("system_time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("start_time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("parent", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pgroup", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("threads", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("nice", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  ColumnAliasSet columnAliases() const override {
    return {
      {"total_size", {"phys_footprint"}},
    };
  }

  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genProcesses(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(processesTablePlugin, "table", "processes");


namespace tables {
osquery::QueryData genNetworkLabels(QueryContext& context);
}

class dockerNetworkLabelsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("key", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genNetworkLabels(context);

    return results;
  }

};


REGISTER(dockerNetworkLabelsTablePlugin, "table", "docker_network_labels");


namespace tables {
osquery::QueryData genACPITables(QueryContext& context);
}

class acpiTablesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("md5", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genACPITables(context);

    return results;
  }

};


REGISTER(acpiTablesTablePlugin, "table", "acpi_tables");


namespace tables {
osquery::QueryData genDNSResolvers(QueryContext& context);
}

class dnsResolversTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("netmask", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("options", BIGINT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genDNSResolvers(context);

    return results;
  }

};


REGISTER(dnsResolversTablePlugin, "table", "dns_resolvers");


namespace tables {
osquery::QueryData genHash(QueryContext& context);
}

class hashTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::INDEX | ColumnOptions::REQUIRED),
      std::make_tuple("directory", TEXT_TYPE, ColumnOptions::REQUIRED),
      std::make_tuple("md5", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sha1", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sha256", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genHash(context);

    return results;
  }

};


REGISTER(hashTablePlugin, "table", "hash");


namespace tables {
osquery::QueryData getAuthorizedKeys(QueryContext& context);
}

class authorizedKeysTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::ADDITIONAL),
      std::make_tuple("algorithm", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("key", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("key_file", TEXT_TYPE, ColumnOptions::INDEX),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::USER_BASED |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::getAuthorizedKeys(context);

    return results;
  }

};


REGISTER(authorizedKeysTablePlugin, "table", "authorized_keys");


namespace tables {
osquery::QueryData genVersion(QueryContext& context);
}

class dockerVersionTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("api_version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("min_api_version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("git_commit", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("go_version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("os", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("arch", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("kernel_version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("build_time", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genVersion(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(dockerVersionTablePlugin, "table", "docker_version");


namespace tables {
osquery::QueryData genContainerStats(QueryContext& context);
}

class dockerContainerStatsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::INDEX | ColumnOptions::REQUIRED),
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pids", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("read", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("preread", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("interval", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("disk_read", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("disk_write", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("num_procs", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_total_usage", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_kernelmode_usage", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_usermode_usage", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("system_cpu_usage", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("online_cpus", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pre_cpu_total_usage", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pre_cpu_kernelmode_usage", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pre_cpu_usermode_usage", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pre_system_cpu_usage", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pre_online_cpus", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("memory_usage", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("memory_max_usage", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("memory_limit", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("network_rx_bytes", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("network_tx_bytes", BIGINT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genContainerStats(context);

    return results;
  }

};


REGISTER(dockerContainerStatsTablePlugin, "table", "docker_container_stats");


namespace tables {
osquery::QueryData genContainerMounts(QueryContext& context);
}

class dockerContainerMountsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("source", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("destination", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("driver", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mode", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("rw", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("propagation", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genContainerMounts(context);

    return results;
  }

};


REGISTER(dockerContainerMountsTablePlugin, "table", "docker_container_mounts");


namespace tables {
osquery::QueryData genGroups(QueryContext& context);
}

class groupsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("gid", BIGINT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("gid_signed", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("groupname", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genGroups(context);

    return results;
  }

};


REGISTER(groupsTablePlugin, "table", "groups");


namespace tables {
osquery::QueryData genDevicePartitions(QueryContext& context);
}

class devicePartitionsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("device", TEXT_TYPE, ColumnOptions::REQUIRED),
      std::make_tuple("partition", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("label", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("offset", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("blocks_size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("blocks", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("inodes", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("flags", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genDevicePartitions(context);

    return results;
  }

};


REGISTER(devicePartitionsTablePlugin, "table", "device_partitions");


namespace tables {
osquery::QueryData genMDDevices(QueryContext& context);
}

class mdDevicesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("device_name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("status", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("raid_level", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("chunk_size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("raid_disks", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("nr_raid_disks", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("working_disks", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("active_disks", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("failed_disks", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("spare_disks", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("superblock_state", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("superblock_version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("superblock_update_time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("bitmap_on_mem", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("bitmap_chunk_size", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("bitmap_external_file", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("recovery_progress", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("recovery_finish", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("recovery_speed", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("resync_progress", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("resync_finish", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("resync_speed", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("reshape_progress", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("reshape_finish", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("reshape_speed", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("check_array_progress", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("check_array_finish", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("check_array_speed", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("unused_devices", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("other", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genMDDevices(context);

    return results;
  }

};


REGISTER(mdDevicesTablePlugin, "table", "md_devices");


namespace tables {
osquery::QueryData genPortageUse(QueryContext& context);
}

class portageUseTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("package", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("use", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genPortageUse(context);

    return results;
  }

};


REGISTER(portageUseTablePlugin, "table", "portage_use");


namespace tables {
osquery::QueryData genMemoryMap(QueryContext& context);
}

class memoryMapTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("start", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("end", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genMemoryMap(context);

    return results;
  }

};


REGISTER(memoryMapTablePlugin, "table", "memory_map");


namespace tables {
osquery::QueryData genMounts(QueryContext& context);
}

class mountsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("device", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("device_alias", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("blocks_size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("blocks", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("blocks_free", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("blocks_available", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("inodes", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("inodes_free", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("flags", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genMounts(context);

    return results;
  }

};


REGISTER(mountsTablePlugin, "table", "mounts");


namespace tables {
osquery::QueryData genInterfaceAddresses(QueryContext& context);
}

class interfaceAddressesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("interface", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mask", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("broadcast", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("point_to_point", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("friendly_name", TEXT_TYPE, ColumnOptions::HIDDEN),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genInterfaceAddresses(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(interfaceAddressesTablePlugin, "table", "interface_addresses");


namespace tables {

class process_events {
 public:
  void genTable(RowYield& yield, QueryContext& context);
};
}

class processEventsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("pid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mode", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cmdline", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cmdline_size", BIGINT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("env", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("env_count", BIGINT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("env_size", BIGINT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("cwd", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("auid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("euid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("gid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("egid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("owner_uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("owner_gid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("atime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mtime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ctime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("btime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("overflows", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("parent", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uptime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("status", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("eid", TEXT_TYPE, ColumnOptions::HIDDEN),
    };
  }


  ColumnAliasSet columnAliases() const override {
    return {
      {"env", {"environment"}},
      {"env_count", {"environment_count"}},
      {"env_size", {"environment_size"}},
      {"atime", {"access_time"}},
      {"mtime", {"modify_time"}},
      {"ctime", {"change_time"}},
      {"btime", {"create_time"}},
    };
  }

  TableAttributes attributes() const override {
    return       TableAttributes::EVENT_BASED |      TableAttributes::NONE;
  }

  bool usesGenerator() const override { return true; }

  void generator(RowYield& yield, QueryContext& context) override {
    if (EventFactory::exists(getName())) {
      auto subscriber = EventFactory::getEventSubscriber(getName());
      return subscriber->genTable(yield, context);
    } else {
      LOG(ERROR) << "Subscriber table missing: " << getName();
    }
  }

};


REGISTER(processEventsTablePlugin, "table", "process_events");


namespace tables {
osquery::QueryData genOSVersion(QueryContext& context);
}

class osVersionTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("major", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("minor", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("patch", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("build", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("platform", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("platform_like", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("codename", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genOSVersion(context);

    return results;
  }

};


REGISTER(osVersionTablePlugin, "table", "os_version");


namespace tables {
osquery::QueryData getKnownHostsKeys(QueryContext& context);
}

class knownHostsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::ADDITIONAL),
      std::make_tuple("key", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("key_file", TEXT_TYPE, ColumnOptions::INDEX),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::USER_BASED |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::getKnownHostsKeys(context);

    return results;
  }

};


REGISTER(knownHostsTablePlugin, "table", "known_hosts");


namespace tables {
osquery::QueryData genMagicData(QueryContext& context);
}

class magicTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::REQUIRED),
      std::make_tuple("data", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mime_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mime_encoding", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genMagicData(context);

    return results;
  }

};


REGISTER(magicTablePlugin, "table", "magic");


namespace tables {
osquery::QueryData genPrometheusMetrics(QueryContext& context);
}

class prometheusMetricsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("target_name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("metric_name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("metric_value", DOUBLE_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("timestamp_ms", BIGINT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genPrometheusMetrics(context);

    return results;
  }

};


REGISTER(prometheusMetricsTablePlugin, "table", "prometheus_metrics");


namespace tables {

class socket_events {
 public:
  void genTable(RowYield& yield, QueryContext& context);
};
}

class socketEventsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("action", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("fd", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("auid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("success", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("family", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("protocol", INTEGER_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("local_address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("remote_address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("local_port", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("remote_port", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("socket", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uptime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("eid", TEXT_TYPE, ColumnOptions::HIDDEN),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::EVENT_BASED |      TableAttributes::NONE;
  }

  bool usesGenerator() const override { return true; }

  void generator(RowYield& yield, QueryContext& context) override {
    if (EventFactory::exists(getName())) {
      auto subscriber = EventFactory::getEventSubscriber(getName());
      return subscriber->genTable(yield, context);
    } else {
      LOG(ERROR) << "Subscriber table missing: " << getName();
    }
  }

};


REGISTER(socketEventsTablePlugin, "table", "socket_events");


namespace tables {
osquery::QueryData portagePackages(QueryContext& context);
}

class portagePackagesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("package", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("slot", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("build_time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("repository", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("eapi", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("world", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::portagePackages(context);

    return results;
  }

};


REGISTER(portagePackagesTablePlugin, "table", "portage_packages");


namespace tables {

class SyslogEventSubscriber {
 public:
  void genTable(RowYield& yield, QueryContext& context);
};
}

class syslogEventsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("datetime", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("host", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("severity", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("facility", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("tag", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("message", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("eid", TEXT_TYPE, ColumnOptions::HIDDEN),
    };
  }

  std::vector<std::string> aliases() const override {
    return {
      "syslog",
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::EVENT_BASED |      TableAttributes::NONE;
  }

  bool usesGenerator() const override { return true; }

  void generator(RowYield& yield, QueryContext& context) override {
    if (EventFactory::exists(getName())) {
      auto subscriber = EventFactory::getEventSubscriber(getName());
      return subscriber->genTable(yield, context);
    } else {
      LOG(ERROR) << "Subscriber table missing: " << getName();
    }
  }

};


REGISTER(syslogEventsTablePlugin, "table", "syslog_events");


namespace tables {
osquery::QueryData genCpuTime(QueryContext& context);
}

class cpuTimeTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("core", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("user", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("nice", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("system", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("idle", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("iowait", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("irq", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("softirq", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("steal", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("guest", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("guest_nice", BIGINT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genCpuTime(context);

    return results;
  }

};


REGISTER(cpuTimeTablePlugin, "table", "cpu_time");


namespace tables {
osquery::QueryData genFDEStatus(QueryContext& context);
}

class diskEncryptionTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uuid", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("encrypted", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uid", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("user_uuid", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genFDEStatus(context);

    return results;
  }

};


REGISTER(diskEncryptionTablePlugin, "table", "disk_encryption");


namespace tables {
osquery::QueryData genSMBIOSTables(QueryContext& context);
}

class smbiosTablesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("number", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("handle", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("header_size", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("md5", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genSMBIOSTables(context);

    return results;
  }

};


REGISTER(smbiosTablesTablePlugin, "table", "smbios_tables");


namespace tables {
osquery::QueryData genModelSpecificRegister(QueryContext& context);
}

class msrTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("processor_number", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("turbo_disabled", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("turbo_ratio_limit", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("platform_info", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("perf_ctl", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("perf_status", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("feature_control", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("rapl_power_limit", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("rapl_energy_status", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("rapl_power_units", BIGINT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genModelSpecificRegister(context);

    return results;
  }

};


REGISTER(msrTablePlugin, "table", "msr");


namespace tables {
osquery::QueryData genLoadAverage(QueryContext& context);
}

class loadAverageTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("period", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("average", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genLoadAverage(context);

    return results;
  }

};


REGISTER(loadAverageTablePlugin, "table", "load_average");


namespace tables {

class yara_events {
 public:
  void genTable(RowYield& yield, QueryContext& context);
};
}

class yaraEventsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("target_path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("category", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("action", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("transaction_id", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("matches", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("count", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("strings", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("tags", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("eid", TEXT_TYPE, ColumnOptions::HIDDEN),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::EVENT_BASED |      TableAttributes::NONE;
  }

  bool usesGenerator() const override { return true; }

  void generator(RowYield& yield, QueryContext& context) override {
    if (EventFactory::exists(getName())) {
      auto subscriber = EventFactory::getEventSubscriber(getName());
      return subscriber->genTable(yield, context);
    } else {
      LOG(ERROR) << "Subscriber table missing: " << getName();
    }
  }

};


REGISTER(yaraEventsTablePlugin, "table", "yara_events");


namespace tables {
osquery::QueryData genSystemInfo(QueryContext& context);
}

class systemInfoTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("hostname", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uuid", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_subtype", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_brand", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_physical_cores", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cpu_logical_cores", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("physical_memory", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("hardware_vendor", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("hardware_model", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("hardware_version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("hardware_serial", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("computer_name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("local_hostname", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genSystemInfo(context);

    return results;
  }

};


REGISTER(systemInfoTablePlugin, "table", "system_info");


namespace tables {
osquery::QueryData genPythonPackages(QueryContext& context);
}

class pythonPackagesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("summary", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("author", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("license", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genPythonPackages(context);

    return results;
  }

};


REGISTER(pythonPackagesTablePlugin, "table", "python_packages");


namespace tables {
osquery::QueryData genNetworks(QueryContext& context);
}

class dockerNetworksTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("driver", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("created", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("enable_ipv6", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("subnet", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("gateway", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genNetworks(context);

    return results;
  }

};


REGISTER(dockerNetworksTablePlugin, "table", "docker_networks");


namespace tables {
osquery::QueryData genKernelIntegrity(QueryContext& context);
}

class kernelIntegrityTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("sycall_addr_modified", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("text_segment_hash", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::KERNEL_REQUIRED |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genKernelIntegrity(context);

    return results;
  }

};


REGISTER(kernelIntegrityTablePlugin, "table", "kernel_integrity");


namespace tables {
osquery::QueryData genListeningPorts(QueryContext& context);
}

class listeningPortsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("pid", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("protocol", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("family", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("address", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genListeningPorts(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(listeningPortsTablePlugin, "table", "listening_ports");


namespace tables {
osquery::QueryData genCarbonBlackInfo(QueryContext& context);
}

class carbonBlackInfoTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("sensor_id", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("config_name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_store_files", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_module_loads", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_module_info", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_file_mods", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_reg_mods", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_net_conns", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_processes", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_cross_processes", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_emet_events", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_data_file_writes", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_process_user_context", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collect_sensor_operations", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("log_file_disk_quota_mb", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("log_file_disk_quota_percentage", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("protection_disabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sensor_ip_addr", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sensor_backend_server", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("event_queue", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("binary_queue", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }

  std::vector<std::string> aliases() const override {
    return {
      "cb_info",
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genCarbonBlackInfo(context);

    return results;
  }

};


REGISTER(carbonBlackInfoTablePlugin, "table", "carbon_black_info");


namespace tables {
osquery::QueryData genProcessEnvs(QueryContext& context);
}

class processEnvsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("pid", INTEGER_TYPE, ColumnOptions::INDEX),
      std::make_tuple("key", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genProcessEnvs(context);

    return results;
  }

};


REGISTER(processEnvsTablePlugin, "table", "process_envs");


namespace tables {
osquery::QueryData genEtcServices(QueryContext& context);
}

class etcServicesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("protocol", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("aliases", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("comment", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genEtcServices(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(etcServicesTablePlugin, "table", "etc_services");


namespace tables {
osquery::QueryData genUSBDevices(QueryContext& context);
}

class usbDevicesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("usb_address", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("usb_port", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("vendor", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("vendor_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("model", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("model_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("serial", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("class", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("subclass", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("protocol", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("removable", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genUSBDevices(context);

    return results;
  }

};


REGISTER(usbDevicesTablePlugin, "table", "usb_devices");


namespace tables {
osquery::QueryData genChromeExtensions(QueryContext& context);
}

class chromeExtensionsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::ADDITIONAL),
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("identifier", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("locale", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("update_url", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("author", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("persistent", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::USER_BASED |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genChromeExtensions(context);

    return results;
  }

};


REGISTER(chromeExtensionsTablePlugin, "table", "chrome_extensions");


namespace tables {
osquery::QueryData genSharedMemory(QueryContext& context);
}

class sharedMemoryTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("shmid", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("owner_uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("creator_uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("creator_pid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("atime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("dtime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ctime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("permissions", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("attached", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("status", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("locked", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genSharedMemory(context);

    return results;
  }

};


REGISTER(sharedMemoryTablePlugin, "table", "shared_memory");


namespace tables {
osquery::QueryData genProcessMemoryMap(QueryContext& context);
}

class processMemoryMapTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("pid", INTEGER_TYPE, ColumnOptions::INDEX),
      std::make_tuple("start", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("end", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("permissions", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("offset", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("device", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("inode", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pseudo", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genProcessMemoryMap(context);

    return results;
  }

};


REGISTER(processMemoryMapTablePlugin, "table", "process_memory_map");


namespace tables {
osquery::QueryData genEc2InstanceTags(QueryContext& context);
}

class ec2InstanceTagsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("instance_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("key", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genEc2InstanceTags(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(ec2InstanceTagsTablePlugin, "table", "ec2_instance_tags");


namespace tables {
osquery::QueryData genShellHistory(QueryContext& context);
}

class shellHistoryTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::ADDITIONAL),
      std::make_tuple("time", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("command", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("history_file", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::USER_BASED |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genShellHistory(context);

    return results;
  }

};


REGISTER(shellHistoryTablePlugin, "table", "shell_history");


namespace tables {
osquery::QueryData genBlockDevs(QueryContext& context);
}

class blockDevicesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("parent", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("vendor", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("model", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("block_size", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uuid", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("label", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genBlockDevs(context);

    return results;
  }

};


REGISTER(blockDevicesTablePlugin, "table", "block_devices");


namespace tables {
osquery::QueryData genContainerPorts(QueryContext& context);
}

class dockerContainerPortsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("port", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("host_ip", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("host_port", INTEGER_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genContainerPorts(context);

    return results;
  }

};


REGISTER(dockerContainerPortsTablePlugin, "table", "docker_container_ports");


namespace tables {

class user_events {
 public:
  void genTable(RowYield& yield, QueryContext& context);
};
}

class userEventsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("auid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pid", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("message", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("terminal", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("time", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("uptime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("eid", TEXT_TYPE, ColumnOptions::HIDDEN),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::EVENT_BASED |      TableAttributes::NONE;
  }

  bool usesGenerator() const override { return true; }

  void generator(RowYield& yield, QueryContext& context) override {
    if (EventFactory::exists(getName())) {
      auto subscriber = EventFactory::getEventSubscriber(getName());
      return subscriber->genTable(yield, context);
    } else {
      LOG(ERROR) << "Subscriber table missing: " << getName();
    }
  }

};


REGISTER(userEventsTablePlugin, "table", "user_events");


namespace tables {
osquery::QueryData genEtcHosts(QueryContext& context);
}

class etcHostsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("hostnames", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }

  std::vector<std::string> aliases() const override {
    return {
      "hosts",
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genEtcHosts(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(etcHostsTablePlugin, "table", "etc_hosts");


namespace tables {
osquery::QueryData genSudoers(QueryContext& context);
}

class sudoersTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("header", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("rule_details", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genSudoers(context);

    return results;
  }

};


REGISTER(sudoersTablePlugin, "table", "sudoers");


namespace tables {
osquery::QueryData genOpenFiles(QueryContext& context);
}

class processOpenFilesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("pid", BIGINT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("fd", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genOpenFiles(context);

    return results;
  }

};


REGISTER(processOpenFilesTablePlugin, "table", "process_open_files");


namespace tables {
osquery::QueryData genSystemControls(QueryContext& context);
}

class systemControlsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("oid", TEXT_TYPE, ColumnOptions::ADDITIONAL),
      std::make_tuple("subsystem", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("current_value", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("config_value", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genSystemControls(context);

    return results;
  }

};


REGISTER(systemControlsTablePlugin, "table", "system_controls");


namespace tables {
osquery::QueryData genDeviceHash(QueryContext& context);
}

class deviceHashTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("device", TEXT_TYPE, ColumnOptions::REQUIRED),
      std::make_tuple("partition", TEXT_TYPE, ColumnOptions::REQUIRED),
      std::make_tuple("inode", BIGINT_TYPE, ColumnOptions::REQUIRED),
      std::make_tuple("md5", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sha1", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sha256", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genDeviceHash(context);

    return results;
  }

};


REGISTER(deviceHashTablePlugin, "table", "device_hash");


namespace tables {
osquery::QueryData genStartupItems(QueryContext& context);
}

class startupItemsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("args", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("source", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("status", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("username", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genStartupItems(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(startupItemsTablePlugin, "table", "startup_items");


namespace tables {
osquery::QueryData genRoutes(QueryContext& context);
}

class routesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("destination", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("netmask", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("gateway", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("source", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("flags", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("interface", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mtu", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("metric", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genRoutes(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(routesTablePlugin, "table", "routes");


namespace tables {
osquery::QueryData genPlatformInfo(QueryContext& context);
}

class platformInfoTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("vendor", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("date", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("revision", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("address", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("volume_size", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("extra", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genPlatformInfo(context);

    return results;
  }

};


REGISTER(platformInfoTablePlugin, "table", "platform_info");


namespace tables {
osquery::QueryData getMemoryInfo(QueryContext& context);
}

class memoryInfoTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("memory_total", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("memory_free", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("buffers", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("cached", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("swap_cached", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("active", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("inactive", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("swap_total", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("swap_free", BIGINT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::getMemoryInfo(context);

    return results;
  }

};


REGISTER(memoryInfoTablePlugin, "table", "memory_info");


namespace tables {
osquery::QueryData genMDDrives(QueryContext& context);
}

class mdDrivesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("md_device_name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("drive_name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("slot", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("state", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genMDDrives(context);

    return results;
  }

};


REGISTER(mdDrivesTablePlugin, "table", "md_drives");


namespace tables {
osquery::QueryData genImageLabels(QueryContext& context);
}

class dockerImageLabelsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("key", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genImageLabels(context);

    return results;
  }

};


REGISTER(dockerImageLabelsTablePlugin, "table", "docker_image_labels");


namespace tables {
osquery::QueryData genInterfaceDetails(QueryContext& context);
}

class interfaceDetailsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("interface", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mac", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("type", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mtu", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("metric", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("flags", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ipackets", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("opackets", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ibytes", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("obytes", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ierrors", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("oerrors", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("idrops", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("odrops", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("collisions", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("last_change", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("friendly_name", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("description", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("manufacturer", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("connection_id", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("connection_status", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("enabled", INTEGER_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("physical_adapter", INTEGER_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("speed", INTEGER_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("dhcp_enabled", INTEGER_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("dhcp_lease_expires", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("dhcp_lease_obtained", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("dhcp_server", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("dns_domain", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("dns_domain_suffix_search_order", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("dns_host_name", TEXT_TYPE, ColumnOptions::HIDDEN),
      std::make_tuple("dns_server_search_order", TEXT_TYPE, ColumnOptions::HIDDEN),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genInterfaceDetails(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(interfaceDetailsTablePlugin, "table", "interface_details");


namespace tables {
osquery::QueryData genEc2Metadata(QueryContext& context);
}

class ec2InstanceMetadataTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("instance_id", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("instance_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("architecture", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("region", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("availability_zone", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("local_hostname", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("local_ipv4", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("mac", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("security_groups", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("iam_arn", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ami_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("reservation_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("account_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("ssh_public_key", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genEc2Metadata(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(ec2InstanceMetadataTablePlugin, "table", "ec2_instance_metadata");


namespace tables {
osquery::QueryData genKernelInfo(QueryContext& context);
}

class kernelInfoTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("arguments", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("device", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genKernelInfo(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(kernelInfoTablePlugin, "table", "kernel_info");


namespace tables {
osquery::QueryData genImages(QueryContext& context);
}

class dockerImagesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("created", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size_bytes", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("tags", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genImages(context);

    return results;
  }

};


REGISTER(dockerImagesTablePlugin, "table", "docker_images");


namespace tables {
osquery::QueryData genPCIDevices(QueryContext& context);
}

class pciDevicesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("pci_slot", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("pci_class", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("driver", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("vendor", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("vendor_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("model", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("model_id", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genPCIDevices(context);

    return results;
  }

};


REGISTER(pciDevicesTablePlugin, "table", "pci_devices");


namespace tables {
osquery::QueryData genOperaExtensions(QueryContext& context);
}

class operaExtensionsTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::ADDITIONAL),
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("identifier", TEXT_TYPE, ColumnOptions::INDEX),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("locale", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("update_url", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("author", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("persistent", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::USER_BASED |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genOperaExtensions(context);

    return results;
  }

};


REGISTER(operaExtensionsTablePlugin, "table", "opera_extensions");


namespace tables {
osquery::QueryData genCPUID(QueryContext& context);
}

class cpuidTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("feature", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("output_register", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("output_bit", INTEGER_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("input_eax", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    auto results = tables::genCPUID(context);

    return results;
  }

};


REGISTER(cpuidTablePlugin, "table", "cpuid");


namespace tables {
osquery::QueryData genRpmPackages(QueryContext& context);
}

class rpmPackagesTablePlugin : public TablePlugin {
 private:
  TableColumns columns() const override {
    return {
      std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("release", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("source", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("sha1", TEXT_TYPE, ColumnOptions::DEFAULT),
      std::make_tuple("arch", TEXT_TYPE, ColumnOptions::DEFAULT),
    };
  }


  TableAttributes attributes() const override {
    return       TableAttributes::CACHEABLE |      TableAttributes::NONE;
  }

  QueryData generate(QueryContext& context) override {
    if (isCached(kCacheStep, context)) {
      return getCache();
    }
    auto results = tables::genRpmPackages(context);
    setCache(kCacheStep, kCacheInterval, context, results);

    return results;
  }

};


REGISTER(rpmPackagesTablePlugin, "table", "rpm_packages");



}