/*
 *  Copyright (c) 2014-present, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the BSD-style license found in the
 *  LICENSE file in the root directory of this source tree. An additional grant
 *  of patent rights can be found in the PATENTS file in the same directory.
 *
 */

/*
** This file is generated. Do not modify it manually!
*/

#include <osquery/events.h>
#include <osquery/logger.h>
#include <osquery/tables.h>


namespace osquery {

void registerForeignTables() {


  class sharingPreferencesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("screen_sharing", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("file_sharing", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("printer_sharing", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("remote_login", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("remote_management", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("remote_apple_events", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("internet_sharing", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bluetooth_sharing", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("disc_sharing", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("sharing_preferences",
      std::make_shared<sharingPreferencesTablePlugin>(), false);
  }

  class temperatureSensorsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("key", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("celsius", DOUBLE_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("fahrenheit", DOUBLE_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("temperature_sensors",
      std::make_shared<temperatureSensorsTablePlugin>(), false);
  }

  class wmiCliEventConsumersTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("command_line_template", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("executable_path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("class", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("relative_path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("wmi_cli_event_consumers",
      std::make_shared<wmiCliEventConsumersTablePlugin>(), false);
  }

  class servicesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("service_type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("display_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("status", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("pid", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("start_type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("win32_exit_code", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("service_exit_code", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("module_path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("user_account", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("services",
      std::make_shared<servicesTablePlugin>(), false);
  }

  class managedPoliciesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("domain", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uuid", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("username", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("manual", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("managed_policies",
      std::make_shared<managedPoliciesTablePlugin>(), false);
  }

  class plistTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("key", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("subkey", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::REQUIRED),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("plist",
      std::make_shared<plistTablePlugin>(), false);
  }

  class keychainItemsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("label", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("comment", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("created", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("modified", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("keychain_items",
      std::make_shared<keychainItemsTablePlugin>(), false);
  }

  class appSchemesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("scheme", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("handler", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("external", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("protected", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("app_schemes",
      std::make_shared<appSchemesTablePlugin>(), false);
  }

  class quicklookCacheTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("rowid", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("fs_id", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("volume_id", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("inode", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("mtime", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("label", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("last_hit_date", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("hit_count", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("icon_mode", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("cache_path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("quicklook_cache",
      std::make_shared<quicklookCacheTablePlugin>(), false);
  }

  class diskEventsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("action", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("device", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uuid", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("ejectable", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("mountable", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("writable", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("content", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("media_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("vendor", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("filesystem", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("checksum", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("time", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("eid", TEXT_TYPE, ColumnOptions::HIDDEN),
      };
    }


  ColumnAliasSet columnAliases() const override {
    return {
      {"device", {"bsd_name"}},
    };
  }

    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("disk_events",
      std::make_shared<diskEventsTablePlugin>(), false);
  }

  class xprotectMetaTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("identifier", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("developer_id", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("min_version", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("xprotect_meta",
      std::make_shared<xprotectMetaTablePlugin>(), false);
  }

  class alfExceptionsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("state", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("alf_exceptions",
      std::make_shared<alfExceptionsTablePlugin>(), false);
  }

  class autoexecTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("source", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("autoexec",
      std::make_shared<autoexecTablePlugin>(), false);
  }

  class adConfigTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("domain", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("option", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("ad_config",
      std::make_shared<adConfigTablePlugin>(), false);
  }

  class wifiSurveyTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("interface", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("ssid", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bssid", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("network_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("country_code", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("rssi", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("noise", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("channel", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("channel_width", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("channel_band", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("wifi_survey",
      std::make_shared<wifiSurveyTablePlugin>(), false);
  }

  class sipConfigTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("config_flag", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("enabled_nvram", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("sip_config",
      std::make_shared<sipConfigTablePlugin>(), false);
  }

  class extendedAttributesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::REQUIRED),
        std::make_tuple("directory", TEXT_TYPE, ColumnOptions::REQUIRED),
        std::make_tuple("key", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("base64", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("extended_attributes",
      std::make_shared<extendedAttributesTablePlugin>(), false);
  }

  class appsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bundle_executable", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bundle_identifier", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bundle_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bundle_short_version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bundle_version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bundle_package_type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("environment", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("element", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("compiler", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("development_region", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("display_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("info_string", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("minimum_system_version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("category", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("applescript_enabled", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("copyright", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("last_opened_time", DOUBLE_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("apps",
      std::make_shared<appsTablePlugin>(), false);
  }

  class programsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("install_source", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("language", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("publisher", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uninstall_string", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("install_date", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("identifying_number", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }

    std::vector<std::string> aliases() const override {
      return {
        "programs_and_features",
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("programs",
      std::make_shared<programsTablePlugin>(), false);
  }

  class gatekeeperTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("assessments_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("dev_id_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("opaque_version", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("gatekeeper",
      std::make_shared<gatekeeperTablePlugin>(), false);
  }

  class alfTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("allow_signed_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("firewall_unload", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("global_state", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("logging_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("logging_option", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("stealth_enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("alf",
      std::make_shared<alfTablePlugin>(), false);
  }

  class gatekeeperApprovedAppsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("requirement", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("ctime", DOUBLE_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("mtime", DOUBLE_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("gatekeeper_approved_apps",
      std::make_shared<gatekeeperApprovedAppsTablePlugin>(), false);
  }

  class crashesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("pid", BIGINT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("crash_path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("identifier", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("parent", BIGINT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("responsible", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uid", INTEGER_TYPE, ColumnOptions::ADDITIONAL),
        std::make_tuple("datetime", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("crashed_thread", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("stack_trace", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("exception_type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("exception_codes", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("exception_notes", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("registers", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("crashes",
      std::make_shared<crashesTablePlugin>(), false);
  }

  class patchesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("csname", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("hotfix_id", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("caption", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("fix_comments", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("installed_by", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("install_date", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("installed_on", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("patches",
      std::make_shared<patchesTablePlugin>(), false);
  }

  class timeMachineDestinationsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("alias", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("destination_id", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("consistency_scan_date", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("root_volume_uuid", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bytes_available", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bytes_used", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("encryption", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("time_machine_destinations",
      std::make_shared<timeMachineDestinationsTablePlugin>(), false);
  }

  class xprotectEntriesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("launch_type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("identity", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("filename", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("filetype", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("optional", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uses_pattern", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("xprotect_entries",
      std::make_shared<xprotectEntriesTablePlugin>(), false);
  }

  class signatureTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::REQUIRED),
        std::make_tuple("signed", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("identifier", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("cdhash", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("team_identifier", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("authority", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("signature",
      std::make_shared<signatureTablePlugin>(), false);
  }

  class appcompatShimsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("executable", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("install_time", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("sdb_id", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("appcompat_shims",
      std::make_shared<appcompatShimsTablePlugin>(), false);
  }

  class virtualMemoryInfoTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("free", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("active", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("inactive", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("speculative", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("throttled", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("wired", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("purgeable", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("faults", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("copy", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("zero_fill", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("reactivated", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("purged", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("file_backed", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("anonymous", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uncompressed", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("compressor", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("decompressed", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("compressed", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("page_ins", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("page_outs", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("swap_ins", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("swap_outs", BIGINT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("virtual_memory_info",
      std::make_shared<virtualMemoryInfoTablePlugin>(), false);
  }

  class authorizationMechanismsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("label", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("plugin", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("mechanism", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("privileged", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("entry", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("authorization_mechanisms",
      std::make_shared<authorizationMechanismsTablePlugin>(), false);
  }

  class preferencesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("domain", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("key", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("subkey", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("forced", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("username", TEXT_TYPE, ColumnOptions::ADDITIONAL),
        std::make_tuple("host", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("preferences",
      std::make_shared<preferencesTablePlugin>(), false);
  }

  class driversTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("device_id", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("device_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("image", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("service", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("service_key", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("inf", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("class", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("provider", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("manufacturer", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("driver_key", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("date", BIGINT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("drivers",
      std::make_shared<driversTablePlugin>(), false);
  }

  class wmiFilterConsumerBindingTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("consumer", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("filter", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("class", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("relative_path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("wmi_filter_consumer_binding",
      std::make_shared<wmiFilterConsumerBindingTablePlugin>(), false);
  }

  class safariExtensionsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::ADDITIONAL),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("identifier", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("sdk", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("update_url", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("author", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("developer_id", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("safari_extensions",
      std::make_shared<safariExtensionsTablePlugin>(), false);
  }

  class sharedResourcesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("install_date", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("status", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("allow_maximum", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("maximum_allowed", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("type", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("shared_resources",
      std::make_shared<sharedResourcesTablePlugin>(), false);
  }

  class fanSpeedSensorsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("fan", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("actual", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("min", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("max", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("target", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("fan_speed_sensors",
      std::make_shared<fanSpeedSensorsTablePlugin>(), false);
  }

  class smcKeysTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("key", TEXT_TYPE, ColumnOptions::INDEX | ColumnOptions::ADDITIONAL),
        std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("size", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("hidden", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("smc_keys",
      std::make_shared<smcKeysTablePlugin>(), false);
  }

  class iokitDevicetreeTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("class", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("id", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("parent", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("device_path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("service", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("busy_state", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("retain_count", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("depth", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("iokit_devicetree",
      std::make_shared<iokitDevicetreeTablePlugin>(), false);
  }

  class keychainAclsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("keychain_path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("authorizations", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("label", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("keychain_acls",
      std::make_shared<keychainAclsTablePlugin>(), false);
  }

  class kernelPanicsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("time", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("registers", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("frame_backtrace", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("module_backtrace", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("dependencies", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("os_version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("kernel_version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("system_model", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uptime", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("last_loaded", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("last_unloaded", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("kernel_panics",
      std::make_shared<kernelPanicsTablePlugin>(), false);
  }

  class powerSensorsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("key", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("category", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("power_sensors",
      std::make_shared<powerSensorsTablePlugin>(), false);
  }

  class sandboxesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("label", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("user", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("build_id", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bundle_path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("sandboxes",
      std::make_shared<sandboxesTablePlugin>(), false);
  }

  class wmiScriptEventConsumersTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("scripting_engine", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("script_file_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("script_text", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("class", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("relative_path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("wmi_script_event_consumers",
      std::make_shared<wmiScriptEventConsumersTablePlugin>(), false);
  }

  class alfExplicitAuthsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("process", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("alf_explicit_auths",
      std::make_shared<alfExplicitAuthsTablePlugin>(), false);
  }

  class certificatesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("common_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("subject", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("issuer", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("ca", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("self_signed", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("not_valid_before", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("not_valid_after", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("signing_algorithm", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("key_algorithm", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("key_strength", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("key_usage", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("subject_key_id", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("authority_key_id", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("sha1", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("certificates",
      std::make_shared<certificatesTablePlugin>(), false);
  }

  class ieExtensionsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("registry_path", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::INDEX),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("ie_extensions",
      std::make_shared<ieExtensionsTablePlugin>(), false);
  }

  class processFileEventsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("action", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("pid", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("parent", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("euid", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("gid", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("egid", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("mode", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("owner_uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("owner_gid", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("atime", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("mtime", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("ctime", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("time", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uptime", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("eid", TEXT_TYPE, ColumnOptions::HIDDEN),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("process_file_events",
      std::make_shared<processFileEventsTablePlugin>(), false);
  }

  class launchdTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("label", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("program", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("run_at_load", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("keep_alive", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("on_demand", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("disabled", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("username", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("groupname", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("stdout_path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("stderr_path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("start_interval", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("program_arguments", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("watch_paths", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("queue_directories", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("inetd_compatibility", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("start_on_mount", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("root_directory", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("working_directory", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("process_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("launchd",
      std::make_shared<launchdTablePlugin>(), false);
  }

  class wifiStatusTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("interface", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("ssid", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("bssid", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("network_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("country_code", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("security_type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("rssi", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("noise", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("channel", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("channel_width", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("channel_band", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("transmit_rate", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("mode", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("wifi_status",
      std::make_shared<wifiStatusTablePlugin>(), false);
  }

  class iokitRegistryTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("class", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("id", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("parent", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("busy_state", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("retain_count", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("depth", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("iokit_registry",
      std::make_shared<iokitRegistryTablePlugin>(), false);
  }

  class xprotectReportsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("user_action", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("time", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("xprotect_reports",
      std::make_shared<xprotectReportsTablePlugin>(), false);
  }

  class registryTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("key", TEXT_TYPE, ColumnOptions::ADDITIONAL),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("data", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("mtime", BIGINT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("registry",
      std::make_shared<registryTablePlugin>(), false);
  }

  class pkgPackagesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("flatsize", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("arch", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("pkg_packages",
      std::make_shared<pkgPackagesTablePlugin>(), false);
  }

  class kernelExtensionsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("idx", INTEGER_TYPE, ColumnOptions::INDEX),
        std::make_tuple("refs", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("linked_against", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("kernel_extensions",
      std::make_shared<kernelExtensionsTablePlugin>(), false);
  }

  class chocolateyPackagesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("summary", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("author", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("license", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("chocolatey_packages",
      std::make_shared<chocolateyPackagesTablePlugin>(), false);
  }

  class packageBomTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("filepath", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uid", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("gid", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("mode", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("size", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("modified_time", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::REQUIRED),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("package_bom",
      std::make_shared<packageBomTablePlugin>(), false);
  }

  class timeMachineBackupsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("destination_id", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("backup_date", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("time_machine_backups",
      std::make_shared<timeMachineBackupsTablePlugin>(), false);
  }

  class wifiNetworksTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("ssid", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("network_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("security_type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("last_connected", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("passpoint", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("possibly_hidden", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("roaming", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("roaming_profile", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("captive_portal", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("auto_login", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("temporarily_disabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("disabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("wifi_networks",
      std::make_shared<wifiNetworksTablePlugin>(), false);
  }

  class nvramTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("nvram",
      std::make_shared<nvramTablePlugin>(), false);
  }

  class authorizationsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("label", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("modified", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("allow_root", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("timeout", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("tries", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("authenticate_user", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("shared", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("comment", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("created", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("class", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("session_owner", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("authorizations",
      std::make_shared<authorizationsTablePlugin>(), false);
  }

  class fbsdKmodsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("size", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("refs", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("address", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("fbsd_kmods",
      std::make_shared<fbsdKmodsTablePlugin>(), false);
  }

  class nfsSharesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("share", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("options", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("readonly", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("nfs_shares",
      std::make_shared<nfsSharesTablePlugin>(), false);
  }

  class launchdOverridesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("label", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("key", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("value", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("launchd_overrides",
      std::make_shared<launchdOverridesTablePlugin>(), false);
  }

  class scheduledTasksTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("action", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("state", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("hidden", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("last_run_time", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("next_run_time", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("last_run_message", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("last_run_code", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("scheduled_tasks",
      std::make_shared<scheduledTasksTablePlugin>(), false);
  }

  class windowsEventsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("time", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("datetime", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("source", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("provider_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("provider_guid", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("eventid", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("task", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("level", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("keywords", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("data", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("eid", TEXT_TYPE, ColumnOptions::HIDDEN),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("windows_events",
      std::make_shared<windowsEventsTablePlugin>(), false);
  }

  class deviceFirmwareTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("type", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("device", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("device_firmware",
      std::make_shared<deviceFirmwareTablePlugin>(), false);
  }

  class homebrewPackagesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("homebrew_packages",
      std::make_shared<homebrewPackagesTablePlugin>(), false);
  }

  class packageReceiptsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("package_id", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("package_filename", TEXT_TYPE, ColumnOptions::INDEX | ColumnOptions::HIDDEN),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("location", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("install_time", DOUBLE_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("installer_name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::ADDITIONAL),
      };
    }

    std::vector<std::string> aliases() const override {
      return {
        "packages",
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("package_receipts",
      std::make_shared<packageReceiptsTablePlugin>(), false);
  }

  class alfServicesTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("service", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("process", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("state", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("alf_services",
      std::make_shared<alfServicesTablePlugin>(), false);
  }

  class eventTapsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("enabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("event_tap_id", INTEGER_TYPE, ColumnOptions::INDEX),
        std::make_tuple("event_tapped", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("process_being_tapped", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("tapping_process", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("event_taps",
      std::make_shared<eventTapsTablePlugin>(), false);
  }

  class sharedFoldersTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("shared_folders",
      std::make_shared<sharedFoldersTablePlugin>(), false);
  }

  class aslTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("time", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("time_nano_sec", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("host", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("sender", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("facility", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("pid", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("gid", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("level", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("message", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("ref_pid", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("ref_proc", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("extra", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("asl",
      std::make_shared<aslTablePlugin>(), false);
  }

  class wmiEventFiltersTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("query", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("query_language", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("class", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("relative_path", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("wmi_event_filters",
      std::make_shared<wmiEventFiltersTablePlugin>(), false);
  }

  class browserPluginsTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("uid", BIGINT_TYPE, ColumnOptions::ADDITIONAL),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("identifier", TEXT_TYPE, ColumnOptions::INDEX),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("sdk", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("description", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("development_region", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("native", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("path", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("disabled", INTEGER_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("browser_plugins",
      std::make_shared<browserPluginsTablePlugin>(), false);
  }

  class packageInstallHistoryTablePlugin : public TablePlugin {
   private:
    TableColumns columns() const override {
      return {
        std::make_tuple("package_id", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("time", INTEGER_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("name", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("version", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("source", TEXT_TYPE, ColumnOptions::DEFAULT),
        std::make_tuple("content_type", TEXT_TYPE, ColumnOptions::DEFAULT),
      };
    }


    QueryData generate(QueryContext& request) override { return QueryData(); }
  };

  {
    auto registry = RegistryFactory::get().registry("table");
    registry->add("package_install_history",
      std::make_shared<packageInstallHistoryTablePlugin>(), false);
  }


}

}